@@ -384,8 +384,9 @@ class ExtendedMinimizer(AbstractObject):
         minimizer_algo = self.config['minimizer_algo']
 
         status = self.minimizer.minimize(minimizer_type, minimizer_algo)
-        while ((status != 0) and (status!=1) and (strategy<2) and (retry>0)):
-            strategy += 1
+        while ((status != 0) and (status!=1) and (retry>0)):
+            if (strategy < 2):
+                strategy += 1
             retry -= 1
             self.stdout.error('ExtendedMinimizer::robust_minimize("{}") fit failed with status {}. '
                           'Retrying with strategy {}'.format(self.name, status, strategy))
