@@ -91,9 +91,22 @@ class XMLWSModifier(XMLWSBase):
             "map"        : _actions.pop("map", []),
             "define"     : _actions.pop("define", []),
             "redefine"   : _actions.pop("redefine", []),
-            "constraint" : _actions.pop("constraint", []),
-            "asimov"     : _actions.pop("asimov", [])
+            "asimov"     : _actions.pop("asimov", []),
+            "constraint" : []
         }
+        
+        # handle constraint definition
+        constraints = _actions.pop("constraint", [])
+        for constraint_dict in constraints:
+            constraint_node = {
+                'attrib': constraint_dict
+            }
+            expr = self._get_node_attrib(constraint_node, "Name", required=True)
+            constr_expr = self.parse_constr_expression(expr, constraint_node)
+            if constr_expr['file'] is None:
+                actions['define'].append(expr)
+            actions['constraint'].append(constr_expr)
+
         _rename = _actions.pop("rename", {})
         rename = {
             "workspace"    : _rename.pop("workspace", {}),
@@ -197,11 +210,9 @@ class XMLWSModifier(XMLWSBase):
         if old_name in self.actions['rename'][target]:
             raise RuntimeError(f"the {target.replace('_','')} \"{old_name}\" is renamed more than once")
         self.actions['rename'][target][old_name] = new_name
-            
+    
     def parse_constr_expression(self, expr:str, node:Dict):
-        pdf_name, obj_type = self._get_object_name_and_type_from_expr(expr)
-        if obj_type != WSObjectType.CONSTRAINT:
-            raise RuntimeError(f"invalid constraint expression: {expr}")
+        pdf_name, _ = self._get_object_name_and_type_from_expr(expr)
         nuis_name = self._get_node_attrib(node, "NP", required=False, default=[], dtype="str_list")
         glob_name = self._get_node_attrib(node, "GO", required=False, default=[], dtype="str_list")
         filename  = self._get_node_attrib(node, "FileName", required=False)
@@ -269,6 +280,9 @@ class XMLWSModifier(XMLWSBase):
         # nothing is done
         if not flag:
             self.stdout.info("INFO: No objects are defined.")
+            
+        # need to import datasets first so the variables are also imported into the workspace
+        self.import_datasets(ws_orig, ws_tmp)
         
         title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
         self.stdout.info(title_str)
@@ -278,17 +292,17 @@ class XMLWSModifier(XMLWSBase):
             self.stdout.info("INFO: No variables are renamed.")
         
         title_str = format_delimiter_enclosed_text("Step 4: Making output workspace", "-")
-        self.stdout.info(title_str)        
+        self.stdout.info(title_str)
+
         # create final workspace
         if len(self.actions['constraint']) > 0:
             ws_final = ROOT.RooWorkspace(ws_name)
             sim_pdf = self.remake_simultaneous_pdf(mc_orig, ws_tmp)
             self.import_object(ws_final, sim_pdf)
+            self.import_datasets(ws_orig, ws_final)
         else:
             ws_final = ws_tmp
-        
-        self.import_datasets(ws_orig, ws_final)
-        
+
         mc_final = self.create_model_config(mc_orig, ws_final, rename_map)
         
         self.rename_objects(ws_final, mc_final)
@@ -518,8 +532,8 @@ class XMLWSModifier(XMLWSBase):
         
         return mc
     
-    @staticmethod
-    def extract_ws_variables(ws:ROOT.RooWorkspace, variables:ROOT.RooArgSet, strict:bool=True):
+    @semistaticmethod
+    def extract_ws_variables(self, ws:ROOT.RooWorkspace, variables:ROOT.RooArgSet, strict:bool=True):
         extracted_variables = ROOT.RooArgSet()
         for variable in variables:
             variable_name = variable.GetName()
@@ -529,6 +543,7 @@ class XMLWSModifier(XMLWSBase):
                     raise RuntimeError(f"missing variable {variable_name} in the workspace {ws.GetName()}")
                 else:
                     self.stdout.warning(f"WARNING: No variable {variable_name} found in the workspace {ws.GetName()}")
+                    continue
             extracted_variables.add(extracted_variable, True)
         return extracted_variables
 
@@ -697,9 +712,10 @@ class XMLWSModifier(XMLWSBase):
             raise RuntimeError(f"pdf {pdf_name} does not exist in workspace {new_ws.GetName()}")
         category = pdf.indexCat()
         n_cat = len(category)
+        
         observables = self.extract_ws_variables(new_ws, old_mc.GetObservables(), True)
         nuisance_parameters = self.extract_ws_variables(new_ws, old_mc.GetNuisanceParameters(), False)
-        
+
         pdf_map = {}
         
         for i in range(n_cat):
@@ -715,7 +731,7 @@ class XMLWSModifier(XMLWSBase):
             base_components.add(obs_terms)
             
             # remove constraint pdfs that are no longer needed
-            if len(dis_constraints.getSize() > 0):
+            if dis_constraints.getSize() > 0:
                 constraints = pdf_i.getAllConstraints(observables, nuisance_parameters, True)
                 base_components.add(constraints)
             
@@ -730,12 +746,13 @@ class XMLWSModifier(XMLWSBase):
                 # check whether the current category depends on the constraint pdf
                 for nuis_name in nuis_names:
                     nuis_var = self._get_relevant_variable(nuis_name, new_ws, pdf_i)
-                    if nuis_var is None:
-                        continue
-                    base_components.add(new_pdf)
+                    if nuis_var is not None:
+                        base_components.add(new_pdf)
+                        break
             new_pdf_name = f"{pdf_i.GetName()}__addConstr"
             pdf_map[category_name] = ROOT.RooProdPdf(new_pdf_name, new_pdf_name, base_components)
-        c_pdf_map = ExtendedModel.get_pdf_map(pdf_map)    
+        c_pdf_map = ExtendedModel.get_pdf_map(pdf_map)
+        pdf_name = old_mc.GetPdf().GetName()
         sim_pdf = ROOT.RooSimultaneous(pdf_name, f"{pdf_name}__addConstr", c_pdf_map, category)
         return sim_pdf
     
@@ -836,4 +853,4 @@ class XMLWSModifier(XMLWSBase):
         function = RooMultiVarGaussian(function_name, function_name, obs_arglist, mean_arglist, V)
         self.import_object(ws, function)
         self.stdout.info(f"INFO: Implemented RooMultiVarGaussian \"{function_name}\" with correlation matrix")
-        self.stdout.info(str(py_V))
+        self.stdout.info(str(py_V))
\ No newline at end of file
