@@ -25,7 +25,7 @@ class ParameterisedAsymptoticCLs(ParameterisedRunner):
         super().__init__(file_expr=file_expr, param_expr=param_expr,
                          filter_expr=filter_expr, exclude_expr=exclude_expr,
                          parallel=parallel, timed=True, save_log=save_log,
-                         cache=cache, verbosity=verbosity)
+                         cache=cache, allow_none=False, verbosity=verbosity)
         
         self.attributes = {
             'input_path': input_path,
