@@ -135,4 +135,19 @@ class AbstractPlot:
         if ylim is not None:
             ax.set_ylim(*ylim)
         if xlim is not None:
-            ax.set_xlim(*xlim)
\ No newline at end of file
+            ax.set_xlim(*xlim)
+    
+    def set_axis_range(self, ax, xmin:Optional[float]=None, xmax:Optional[float]=None,
+                       ymin:Optional[float]=None, ymax:Optional[float]=None):
+        xlim = list(ax.get_xlim())
+        ylim = list(ax.get_ylim())
+        if xmin is not None:
+            xlim[0] = xmin
+        if xmax is not None:
+            xlim[1] = xmax
+        if ymin is not None:
+            ylim[0] = ymin
+        if ymax is not None:
+            ylim[1] = ymax
+        ax.set_xlim(*xlim)
+        ax.set_ylim(*ylim)
\ No newline at end of file
